#workshop{font-size:.875rem}.actionbar[data-v-26b92757]{height:3rem}.actionbar+div[data-v-26b92757]{min-height:0}@media (min-width: 992px){.offcanvas-lg[data-v-26b92757]{width:350px;flex-shrink:0}.offcanvas-body[data-v-26b92757]{border-top:0!important}}.list-group-item-title[data-v-26b92757]{line-height:1.2em;max-height:2.4em;overflow:hidden;display:-webkit-box;-webkit-line-clamp:2;-webkit-box-orient:vertical;text-overflow:ellipsis}.list-group .dropdown-toggle[data-v-26b92757]:not(:hover):not(:active){transition:none;background-color:transparent;border-color:transparent}.cm-editor{height:100%}.border-primary[data-v-b532536f]{border-width:2px;border-style:dashed}
