.dropdown-toggle[data-v-92b87841],.dropdown-item[data-v-92b87841],.dropdown-toggle[data-v-87d7ecea],.dropdown-item[data-v-87d7ecea]{white-space:normal}@keyframes blink-animation-b714634c{to{visibility:hidden}}.writing[data-v-b714634c]:after{content:"";width:1ch;height:1em;background:var(--ownai-primary);display:inline-block;animation:blink-animation-b714634c 1s steps(3,start) infinite}.error .error-badge[data-v-b714634c]{display:inline-block!important}.card-body[data-v-b714634c]{white-space:break-spaces}
