<div class="dropdown flex-shrink-0">
  <a
    href="#"
    class="d-flex align-items-center link-dark text-decoration-none dropdown-toggle"
    data-bs-toggle="dropdown"
    aria-expanded="false"
  >
    <div
      class="rounded-circle bg-primary d-flex justify-content-center align-items-center"
      style="width: 2rem; height: 2rem;"
    >
      <span class="text-white">{{ g.user['username'][0] }}</span>
    </div>
  </a>
  <ul class="dropdown-menu text-small shadow m-0 dropdown-menu-end">
    <li>
      <span class="dropdown-item-text text-secondary"
        >Hello {{ g.user['username'] }}!</span
      >
    </li>
    <li>
      <a class="dropdown-item" href="{{ url_for('auth.logout') }}">Sign out</a>
    </li>
  </ul>
</div>
